/* PIO-D144/D168 Libary Function.

   Author: Golden Wang

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   Use vi or gvim to edit this file and set tabstop=4.

   v 0.0.0  11 Jun 2007 by Golde Wang
     function created for digital I/O and signal interrupt .

*/

#include "piodio.h"

extern char device_module[LINE_SIZE + 1];
extern dev_control_t *dev_ctl;

void piod144_d168_port_config(unsigned int *reg_value, WORD port, boolean io)
{
  if ( ((short)port == 0 && io == 0) || (port == 6 && io == 0) || (port == 12 && io == 0) )
  {
    *reg_value = *reg_value & 0x3e;
  }
  else if ( ((short)port == 0 && io == 1) || (port == 6 && io == 1) || (port == 12 && io == 1))
  {
    *reg_value = *reg_value | 0x01;
  }

  if ( (port == 1 && io == 0) || (port == 7 && io == 0) || (port == 13 && io == 0) )
  {
    *reg_value = *reg_value & 0x3d;
  }
  else if ( (port == 1 && io == 1) || (port == 7 && io == 1) || (port == 13 && io == 1) )
  {
    *reg_value = *reg_value | 0x02;
  }

  if ( (port == 2 && io == 0) || (port == 8 && io == 0) || (port == 14 && io == 0) )
  {
    *reg_value = *reg_value & 0x3b;
  }
  else if ( (port == 2 && io == 1) || (port == 8 && io == 1) || (port == 14 && io == 1) )
  {
    *reg_value = *reg_value | 0x04;
  }

  if ( (port == 3 && io == 0) || (port == 9 && io == 0) || (port == 15 && io == 0) )
  {
    *reg_value = *reg_value & 0x37;
  }
  else if ( (port == 3 && io == 1) || (port == 9 && io == 1) || (port == 15 && io == 1) )
  {
    *reg_value = *reg_value | 0x08;
  }

  if ( (port == 4 && io == 0) || (port == 10 && io == 0) || (port == 16 && io == 0) )
  {
    *reg_value = *reg_value & 0x2f;
  }
  else if ( (port == 4 && io == 1) || (port == 10 && io == 1) || (port == 16 && io == 1) )
  {
    *reg_value = *reg_value | 0x10;
  }

  if ( (port == 5 && io == 0) || (port == 11 && io == 0) || (port == 17 && io == 0) )
  {
    *reg_value = *reg_value & 0x1f;
  }
  else if ( (port == 5 && io == 1) || (port == 11 && io == 1) || (port == 17 && io == 1) )
  {
    *reg_value = *reg_value | 0x20;
  }

  if ( port == 18 && io == 0 )
  {
    *reg_value = *reg_value & 0x06;
  }
  else if ( port == 18 && io == 1 )
  {
    *reg_value = *reg_value | 0x01;
  }

  if ( port == 19 && io == 0 )
  {
    *reg_value = *reg_value & 0x05;
  }
  else if ( port == 19 && io == 1 )
  {
    *reg_value = *reg_value | 0x02;
  }

  if ( port == 20 && io == 0 )
  {
    *reg_value = *reg_value & 0x03;
  }
  else if ( port == 20 && io == 1 )
  {
    *reg_value = *reg_value | 0x04;
  }
}

WORD PIOD144_168_DeviceInit(WORD fd, WORD port_num)
{
	int i;	
	ixpio_reg_t reg;

	for(i = 0; i <= port_num; i++)
	{
		/* select io port */
		reg.id = IXPIO_AIOPCR;       //0xc4
		reg.value = i;                       /* port i actived */

		if (PIODA_OutputByte(fd, &reg)) 
			return FAILURE;

		reg.id = IXPIO_8DR;
		reg.value = 0;

		if (PIODA_OutputByte(fd, &reg))
			return FAILURE;
	}

	return SUCCESS;
}

/* initial piod144 port property */
WORD PIOD144_168_PortDirCfs(WORD fd, WORD port, boolean io)
{
  ixpio_reg_t reg;

  
  if( port <= 5)
  {
    reg.id = IXPIO_IOSCRA;      /* port0 ~ port5 */
    reg.value = (dev_ctl->pconf[LEVEL_A].init)? dev_ctl->pconf[LEVEL_A].status : 0x3f;
    piod144_d168_port_config(&reg.value, port, io);

    if (PIODA_OutputByte(fd, &reg)) return FAILURE;

    dev_ctl->pconf[LEVEL_A].status = reg.value;
    dev_ctl->pconf[LEVEL_A].id = reg.id;
    if ( !dev_ctl->pconf[LEVEL_A].init ) dev_ctl->pconf[LEVEL_A].init = 1;
  }
  else if( port > 5 && port <= 11 )
  {
    reg.id = IXPIO_IOSCRB;      /* port6 ~ port11 */
    reg.value = (dev_ctl->pconf[LEVEL_B].init)? dev_ctl->pconf[LEVEL_B].status : 0x3f;
    piod144_d168_port_config(&reg.value, port, io);
    if (PIODA_OutputByte(fd, &reg)) return FAILURE;

    dev_ctl->pconf[LEVEL_B].status = reg.value;
    dev_ctl->pconf[LEVEL_B].id = reg.id;
    if ( !dev_ctl->pconf[LEVEL_B].init ) dev_ctl->pconf[LEVEL_B].init = 1;
  }
  else if( port > 11 && port <= 17 )
  {
    reg.id = IXPIO_IOSCRC;      /* port12 ~ port17 */
    reg.value = (dev_ctl->pconf[LEVEL_C].init)? dev_ctl->pconf[LEVEL_C].status : 0x3f;
    piod144_d168_port_config(&reg.value, port, io);
    if (PIODA_OutputByte(fd, &reg)) return FAILURE;

    dev_ctl->pconf[LEVEL_C].status = reg.value;
    dev_ctl->pconf[LEVEL_C].id = reg.id;
    if ( !dev_ctl->pconf[LEVEL_C].init ) dev_ctl->pconf[LEVEL_C].init = 1;
  }
  else if( port > 17 && port <= 20 ) /* only support for PIOD168 */
  {
    reg.id = IXPIO_IOSCRD;      /* port18 ~ port20 */
    reg.value = (dev_ctl->pconf[LEVEL_D].init)? dev_ctl->pconf[LEVEL_D].status : 0x07;
    piod144_d168_port_config(&reg.value, port, io);
    if (PIODA_OutputByte(fd, &reg)) return FAILURE;

    dev_ctl->pconf[LEVEL_D].status = reg.value;
    dev_ctl->pconf[LEVEL_D].id = reg.id;
    if ( !dev_ctl->pconf[LEVEL_D].init ) dev_ctl->pconf[LEVEL_D].init = 1;
  }
  
  return SUCCESS;
}

WORD PIOD144_168_Digital_Output(WORD fd, WORD port, byte data)
{
  ixpio_reg_t reg;

  /* select I/O port */
  reg.id = IXPIO_AIOPCR;
  reg.value = port;

  if (PIODA_OutputByte(fd, &reg)) return PIODA_ACTIVE_PORT_ERROR;
  
  /* ouput data */
  reg.id = IXPIO_8DR;
  reg.value = data;                   

  if (PIODA_OutputByte(fd, &reg)) return PIODA_DIGITAL_OUTPUT_ERROR;
  
  return SUCCESS;
}


WORD PIOD144_168_Digital_Input(WORD fd, WORD port, WORD *di_data)
{
  ixpio_reg_t reg;

  /* select I/O port */
  reg.id = IXPIO_AIOPCR;
  reg.value = port;

  if (PIODA_OutputByte(fd, &reg)) return PIODA_ACTIVE_PORT_ERROR;

  /* input data */
  reg.id = IXPIO_8DR;

  if (PIODA_InputByte(fd, &reg)) return PIODA_DIGITAL_INPUT_ERROR;
 
  *di_data = reg.value;

  return SUCCESS;
}

WORD PIOD144_168_Int_Source_Init(WORD fd, WORD signal, WORD int_source)
{
  ixpio_reg_t reg;
  struct sigaction act_old;

  /* check interrupt sources first */
  if (int_source != PIOD144_168_P2C0 && int_source != PIOD144_168_P2C1 && int_source != PIOD144_168_P2C2 && int_source != PIOD144_168_P2C3 && int_source != PIOD144_168_ALL_INT)
  {
    close(fd);
    sigaction(signal, &act_old, NULL);
    return PIODA_INT_SOURCE_DEFINED_ERROR;
  }

  /* select I/O port */
  reg.id = IXPIO_AIOPCR;
  reg.value = 0x02;

  if (PIODA_OutputByte(fd, &reg))
  {
    sigaction(signal, &act_old, NULL);
    return PIODA_ACTIVE_PORT_ERROR;
  }

  /* configure board interrupt */
  reg.id = IXPIO_IMCR;
  reg.value = int_source;

  if (PIODA_OutputByte(fd, &reg))
  {
    sigaction(signal, &act_old, NULL);
    return PIODA_CONFIGURE_INTERRUPT_ERROR;
  }

  return SUCCESS;
}

