/* PIO-D24/D56 Linux Static Libary.

   Author: Golden Wang

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   Use vi or gvim to edit this file and set tabstop=4.

   v 0.0.0  7  Jul 2007 by Golden Wang
     function created for digital I/O and signal interrupt.

*/

#include "piodio.h"

extern char device_module[LINE_SIZE + 1];
extern dev_control_t *dev_ctl;

WORD PIOD56_24_PortDirCfs(WORD fd, WORD port, boolean io)
{
  ixpio_reg_t reg;
 
  reg.id = IXPIO_PC;

  //default all ports are DI.
  reg.value = (dev_ctl->pconf[LEVEL_A].init) ? dev_ctl->pconf[LEVEL_A].status : 0;

  if ( (short)port == 0 && io == 0 )
  {   
    reg.value = reg.value | 0x01;
  }
  else if ( (short)port == 0 && io == 1 )
  {
    reg.value = reg.value & 0x06;
  }

  if ( port == 1 && io == 0 )
  {
    reg.value = reg.value | 0x02;
  }
  else if ( port == 1 && io == 1 )
  {
    reg.value = reg.value & 0x05;
  }

  if ( port == 2 && io == 0 )
  {
    reg.value = reg.value | 0x04;
  }
  else if ( port == 2 && io == 1 )
  {
    reg.value = reg.value & 0x03;
  }

  /* setup I/O Selection Control Register */
  if (PIODA_OutputByte(fd, &reg)) return FAILURE;

  dev_ctl->pconf[LEVEL_A].status = reg.value;
  dev_ctl->pconf[LEVEL_A].id = reg.id;  
  if ( !dev_ctl->pconf[LEVEL_A].init ) dev_ctl->pconf[LEVEL_A].init = 1;

  return SUCCESS;
}  

WORD PIOD56_24_Digital_Output(WORD fd, WORD port, byte data)
{
  ixpio_reg_t reg;

  WORD PIOD56_24_PORT_ID[3] = { IXPIO_P0, IXPIO_P1, IXPIO_P2 };

  reg.id = PIOD56_24_PORT_ID[port];
  reg.value = data;

  /* setup I/O Selection Control Register */
  if (PIODA_OutputByte(fd, &reg)) return PIODA_DIGITAL_OUTPUT_ERROR;

  return SUCCESS;
}

WORD PIOD56_24_Digital_Input(WORD fd, WORD port, WORD *di_data)
{
  ixpio_reg_t reg;

  WORD PIOD56_24_PORT_ID[3] = { IXPIO_P0, IXPIO_P1, IXPIO_P2 };

  reg.id = PIOD56_24_PORT_ID[port];

  if (PIODA_InputByte(fd, &reg)) return PIODA_DIGITAL_INPUT_ERROR;
  
  *di_data = reg.value;
  
  return SUCCESS;
}

WORD PIOD56_24_Int_Source_Init(WORD fd, WORD signal, WORD int_source)
{
  ixpio_reg_t reg;
  struct sigaction act_old;
  WORD rs;

  if ( !strcmp(device_module,"ixpiod56") || !strcmp(device_module,"ixpiod24") )
  {
    if ( int_source == PIOD56_24_P2C0 || int_source == PIOD56_24_P2C1 || int_source == PIOD56_24_P2C2 || int_source == PIOD56_24_P2C3 || int_source == PIOD56_24_ALL_INT )
    {
      rs = PIODA_PortDirCfs(fd, 2, DIGITAL_INPUT);
    }
    else
    {
      close(fd);
      sigaction(signal, &act_old, NULL);
      return PIODA_INT_SOURCE_DEFINED_ERROR;
    }
  }

  if ( rs )
  {
    sigaction(signal, &act_old, NULL);
    return PIODA_ACTIVE_PORT_ERROR;
  }

  /* configure board interrupt */
  reg.id = IXPIO_IMCR;
  reg.value = int_source;

  if (PIODA_OutputByte(fd, &reg))
  {
    sigaction(signal, &act_old, NULL);
    return PIODA_CONFIGURE_INTERRUPT_ERROR;
  }

  return SUCCESS;
}

