#include "piodio.h"

extern char device_module[LINE_SIZE + 1];
extern WORD channel_record;
extern WORD channel_init;
extern WORD gain_record;
extern WORD gain_init;

// ******************************************************************
//   Function for Analog Output or Input cability
// ******************************************************************

/* PISO-813 Analog Input Voltage */
WORD PISO813_Analog_Input_Voltage(WORD fd, WORD channel, WORD gain, float *fvalue)
{
  ixpio_reg_t pgcr, mcsr, ai;
  
  /* PISO-813 Gain code setting */

  if ( gain_init || (!gain_init && gain_record != gain) ) 
  {
    pgcr.id = IXPIO_PGCR;
    pgcr.value = gain; 

    if ( PIODA_OutputByte(fd, &pgcr) ) return PIODA_SET_GAIN_ERROR;
 
    usleep(12); /* PGA + propagation settling delay */

    if ( gain_init ) gain_init = 0;

    gain_record = gain;
  }

  /* PISO-813 Input channel setting */
  if ( channel_init || (!channel_init && channel_record != channel) )
  {
    mcsr.id = IXPIO_MCSR;
    mcsr.value = channel; 
  
    if ( PIODA_OutputByte(fd, &mcsr) ) return PIODA_SET_CHANNEL_ERROR;
 
    usleep(10); /* multiplexer settling delay */

    if ( channel_init ) channel_init = 0;

    channel_record = gain;
  }

  ai.id = IXPIO_AD;

  if ( PIODA_InputByte(fd, &ai) ) return PIODA_ANALOG_INPUT_VOLTAGE_ERROR;
  *fvalue = (float)ai.value; 
 
  return SUCCESS;
}

